import logging
import sqlparams

logger = logging.getLogger(__name__)


class CursorWrapper:
    def __init__(self, cursor, style, sql_converter):
        self._cursor = cursor

        self._sql_params_converter = sqlparams.SQLParams('named', style, True, True)

        self._sql_converter = sql_converter

        self._statement = None

    def execute(self, query, parameters=None):
        if isinstance(parameters, dict):
            _query, _parameters = self._sql_params_converter.format(query, parameters)
        else:
            _query, _parameters = self._sql_converter(query), parameters

        if query != _query:
            logger.debug(
                'SQL (%s), parameters(%s) has been converted to SQL(%s), parameters(%s)',
                query, parameters, _query, _parameters
            )

        query, parameters = _query, _parameters

        try:
            cursor = self._cursor.cursor
        except AttributeError:
            cursor = self._cursor

        self._statement = query

        return cursor.execute(query, parameters or [])

    def __getattr__(self, attr):
        try:
            return getattr(self._cursor, attr)
        except AttributeError:
            if attr == 'statement':
                return self._statement
            raise



# -*- coding: utf-8 -*-

from sqlalchemy.dialects.postgresql.psycopg2 import PGDialect_psycopg2
from dj_db_conn_pool.core.mixins import PersistentDatabaseWrapperMixin


class PGDatabaseWrapperMixin(PersistentDatabaseWrapperMixin):
    class SQLAlchemyDialect(PGDialect_psycopg2):
        pass



# -*- coding: utf-8 -*-

from django.contrib.gis.db.backends.postgis import base
from sqlalchemy.dialects.postgresql.base import PGDialect
from dj_db_conn_pool.core.mixins import PersistentDatabaseWrapperMixin


class DatabaseWrapper(PersistentDatabaseWrapperMixin, base.DatabaseWrapper):
    class SQLAlchemyDialect(PGDialect):
        pass



# -*- coding: utf-8 -*-

from django.db.backends.postgresql import base
from dj_db_conn_pool.backends.postgresql.mixins import PGDatabaseWrapperMixin


class DatabaseWrapper(PGDatabaseWrapperMixin, base.DatabaseWrapper):
    def get_new_connection(self, conn_params):
        connection = super().get_new_connection(conn_params)

        if not connection.info:
            connection.info = connection.connection.info

        return connection



# encoding: utf-8

import codecs
import shutil
import setuptools

from dj_db_conn_pool import (
    __version__,
    __author__,
    __author_email__,
    __description__
)


def clean(name):
    def decorator(func):
        def smash_the_egg():
            shutil.rmtree(name + '.egg-info', ignore_errors=True)
        return lambda: [fn() for fn in [smash_the_egg, func, smash_the_egg]]
    return decorator


@clean('django_db_connection_pool')
def setup():
    setuptools.setup(
        name='django-db-connection-pool',
        license='MIT',
        version=__version__,
        description=__description__,
        long_description=codecs.open('README.md', encoding='UTF-8').read(),
        long_description_content_type='text/markdown',
        author=__author__,
        author_email=__author_email__,
        url='https://github.com/altairbow/django-db-connection-pool',
        download_url='https://pypi.python.org/pypi/django-db-connection-pool/',
        packages=setuptools.find_packages(),
        include_package_data=True,
        install_requires=[
            'Django>=2.0',
            'SQLAlchemy>=1.4.24',
            'sqlparams>=4.0.0',
        ],
        extras_require={
            'all': [
                'JPype1>=1.3.0',
                'sqlparams>=3.0.0',
                'PyMySQL>=0.9.3',
                'pyodbc>=4.0.34',
                'cx-Oracle>=6.4.1',
                'psycopg2>=2.8.6',
            ],
            'jdbc': ['JPype1>=1.3.0'],
            'mysql': ['PyMySQL>=0.9.3'],
            'odbc': ['pyodbc>=4.0.34'],
            'oracle': ['cx-Oracle>=6.4.1'],
            'postgresql': ['psycopg2>=2.8.6'],
        },
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Environment :: Web Environment',
            'Framework :: Django',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Operating System :: OS Independent',
            'Programming Language :: Python :: 3',
            'Topic :: Software Development :: Libraries :: Python Modules',
        ],
        keywords=['django', 'db', 'database', 'persistent', 'connection', 'pool', 'pooling'],
    )


if __name__ == '__main__':
    setup()
